
#include <string.h>
#include "pathname.h"
#include "helpers.h"
#include "macstuff.h"


#ifdef MACZIP
#include "osdep.h"
#include "macglob.h"

extern MacZipGlobals	MacZip;



const char	ResourceMark[] = "Resource.Frk:";  /* see also macos.c */

/* var used by GetNextDirItem and GetFirstItem */
static short itemIndex = 1;

char *StripPartialDir(char *CompletePath, char *PartialPath,char *FullPath)
{  
char *tmpPtr1 = PartialPath;
char *tmpPtr2 = FullPath;
short	count;
int		result;

if (MacZip.DataForkOnly) 
		{
		for ( ; *tmpPtr1; tmpPtr1++) 
			tmpPtr2++;
		return tmpPtr2;
		}
	

switch (MacZip.MacZipMode)
	{
 	case JohnnyLee:
		{
		for ( ; *tmpPtr1; tmpPtr1++) 
			tmpPtr2++;
		return tmpPtr2;
		break;
		}

	case NewZipMode:
		{			/* determine Fork type */		
		result = strncmp(FullPath,ResourceMark,sizeof(ResourceMark)-2);			
		if (result != 0) 
			{
			MacZip.CurrentFork = DataFork;
			for ( ; *tmpPtr1; tmpPtr1++)    /* data fork  */
				tmpPtr2++;
			 return tmpPtr2;
			}
		else  
			{
			MacZip.CurrentFork = ResourceFork;
			strcpy(CompletePath,ResourceMark);
			for ( ; *tmpPtr1; tmpPtr1++)    /* resource fork  */
				tmpPtr2++; 
			for (count = 0 ; count != sizeof(ResourceMark)-1; count++)    
				tmpPtr2++; 
			strcat(CompletePath,tmpPtr2); 
			return CompletePath;	
			}
		break;
		}
	default: return NULL;  /* function should never reach this point */
	}


 return NULL;    /* function should never reach this point */
}

/* This Func will generate the following warning:
Warning : illegal implicit const pointer conversion
helpers.c line 193   for (VolEnd = FullPath; *VolEnd != ':'; VolEnd++)
*/
unsigned short GetVolumeFromPath(char *FullPath,char *VolumeName)
{  
char *VolEnd, *tmpPtr1, *tmpPtr2 = VolumeName;

for (VolEnd = FullPath; *VolEnd != ':'; VolEnd++)
      ;  
 
for (tmpPtr1 = FullPath; tmpPtr1 != VolEnd; tmpPtr1++)            
	{ 
  	*tmpPtr2 = *tmpPtr1;
  	tmpPtr2++;
  	}

*tmpPtr2 = 0x00;

return (unsigned short) strlen(VolumeName);
}


/*
* This function expands a given partial path to a complete path.
* Path expansions are relative to the running app.
* This function follows the notation:
*	1. relative path:  
* 		a: ":subfolder:filename"  	-> ":currend folder:subfolder:filename"
*		b: "::folder2:filename" 	-> folder2 is beside the currend folder on the same level
*		c: "filename" 				-> in currend folder
*
* A absolute path will be returned.
*/

char *GetCompletePath(char *CompletePath, char *name, FSSpec *Spec,OSErr *err)
{
Boolean hasDirName = false;
char *tmpPtr;
short fullPathLength;

strcpy(CompletePath,MacZip.CurrentPath);

for (tmpPtr = name; *tmpPtr; tmpPtr++)         
	if (*tmpPtr == ':')	hasDirName = true;

if (name[0] != ':')		/* case c: path including volume name or only filename */
	{						
	if (hasDirName) 
		{               /* okey starts with volumename so it must be a complete path */
		strcpy(CompletePath,name);
		fullPathLength = strlen(CompletePath);
		*err = FSpLocationFromFullPath(fullPathLength,CompletePath, Spec);
		return strcpy(CompletePath,name);	
		}
	else
		{	            /* only filename: add cwd and return */ 
		strcat(CompletePath,name);
		fullPathLength = strlen(CompletePath);
		*err = FSpLocationFromFullPath(fullPathLength,CompletePath, Spec);
		return CompletePath;
		}
	}

if ((name[0] == ':') && (name[1] == ':'))	/* it's case b: */
	{
			/* it's not yet implemented; do we really need this case ?*/
	return NULL;
	}
else
	{										/* it's case a: */
	CompletePath[strlen(CompletePath)-1] = 0x00; /* we don't need a second colon */
	strcat(CompletePath,name);
	fullPathLength = strlen(CompletePath);
	*err = FSpLocationFromFullPath(fullPathLength,CompletePath, Spec);
	return CompletePath;
	}

return NULL;
}





char *TruncFilename(char *CompletePath, const char *name, FSSpec *Spec,OSErr *err)
{
char *tmpPtr;
char *dirPtr;
short fullPathLength = 0 ;

strcpy(CompletePath,name);

for (tmpPtr = CompletePath; *tmpPtr; tmpPtr++)         
	if (*tmpPtr == ':')	dirPtr = tmpPtr;

*++dirPtr = 0x00;

fullPathLength = strlen(CompletePath);
printerr("Warning path length exceeds limit: ",fullPathLength >= NAME_MAX ,
		fullPathLength,__LINE__,__FILE__," chars ");

*err = FSpLocationFromFullPath(fullPathLength,CompletePath,Spec);

return CompletePath;
}




char *GetFilename(char *CompletePath, char *name)
{
char *tmpPtr;
char *dirPtr = NULL;


for (tmpPtr = name; *tmpPtr; tmpPtr++)         
	if (*tmpPtr == ':')	dirPtr = tmpPtr;
	
if (dirPtr == NULL) return strcpy(CompletePath,name);
	
dirPtr++;

return strcpy(CompletePath,dirPtr);
}




char *GetFullPathFromID(char *CompletePath, short vRefNum, long dirID, ConstStr255Param name, OSErr *err)
{
FSSpec		spec;
	
	*err = FSMakeFSSpecCompat(vRefNum, dirID, name, &spec);
	printerr("FSMakeFSSpecCompat:",(*err != -43) && (*err != 0),*err,__LINE__,__FILE__,"");
	if ( (*err == noErr) || (*err == fnfErr) )
	{
		return GetFullPathFromSpec(CompletePath,&spec, err);
	}
	
return NULL;
}




char *Real2RfDfFilen(char *RfDfFilen, const char *RealPath, short CurrentFork)
{

if (MacZip.DataForkOnly) /* make no changes */
	{
	MacZip.CurrentFork = DataFork;
	return strcpy(RfDfFilen,RealPath);
	}

switch (MacZip.MacZipMode)
	{
	case JohnnyLee:
		{ 
		strcpy(RfDfFilen,RealPath);
		if (CurrentFork == DataFork) return strcat(RfDfFilen,"d");  /* data-fork  */
		if (CurrentFork == ResourceFork) return strcat(RfDfFilen,"r");  /* resource-fork */
		break;
		}

	case NewZipMode:
		{
		switch (CurrentFork)
			{ 
			case DataFork:
				{
				strcpy(RfDfFilen, RealPath);
				return RfDfFilen;  /* data-fork  */
				break;
				}
			case ResourceFork:
				{
				strcpy(RfDfFilen, ResourceMark);
				strcat(RfDfFilen,RealPath);  /* resource-fork */
				return RfDfFilen;
				break;
				}
			default: 
				{
				printerr("Real2RfDfFilen:",-1,-1,__LINE__,__FILE__,RealPath);  
				return NULL;  /* function should never reach this point */
				}		
			}
		break;
		}
	default:
		{
		printerr("Real2RfDfFilen:",-1,-1,__LINE__,__FILE__,RealPath);  
		return NULL;  /* function should never reach this point */
		}		
	}

printerr("Real2RfDfFilen:",-1,-1,__LINE__,__FILE__,RealPath);  
return NULL;  /* function should never come reach this point */
}





char *RfDfFilen2Real(char *RealFn, const char *RfDfFilen)
{
short 	length;
int 	result;

if (MacZip.DataForkOnly) 
	{
	MacZip.CurrentFork = DataFork;
	return strcpy(RealFn,RfDfFilen);
	}

switch (MacZip.MacZipMode)
	{
	case JohnnyLee:
		{ 
		strcpy(RealFn, RfDfFilen);
		length = strlen(RealFn);       /* determine Fork type */
		if (RealFn[length-1] == 'd') MacZip.CurrentFork = DataFork;
		else MacZip.CurrentFork = ResourceFork;		
		RealFn[length-1] = 0x00;       /* simply cut one char  */
		return RealFn;
		break;
		}

	case NewZipMode:
		{									/* determine Fork type */			
		result = strncmp(RfDfFilen,ResourceMark,sizeof(ResourceMark)-2);
		if (result != 0) 
			{
			MacZip.CurrentFork = DataFork;
			strcpy(RealFn, RfDfFilen);
			return RealFn;  /* data-fork  */
			}
		else  
			{
			MacZip.CurrentFork = ResourceFork;
			strcpy(RealFn, &RfDfFilen[sizeof(ResourceMark)-1]);
			return RealFn;  /* resource-fork */
			}
		break;
		}
	default:
		{
		printerr("RfDfFilen2Real():",-1,-1,__LINE__,__FILE__,RfDfFilen); 
		return NULL;  /* function should never reach this point */
		}
	}

printerr("RfDfFilen2Real():",-1,-1,__LINE__,__FILE__,RfDfFilen); 
return NULL;  /* function should never reach this point */
}
#endif   /*  #ifdef MACZIP */


char *GetAppName(void)
{
ProcessSerialNumber psn;
static Str255       AppName;
ProcessInfoRec      pinfo;
OSErr               err;

GetCurrentProcess(&psn);
pinfo.processName = AppName;
pinfo.processInfoLength = sizeof(pinfo);
pinfo.processAppSpec = NULL;

err = GetProcessInformation(&psn,&pinfo);
AppName[AppName[0]+1] = 0x00;

return (char *)&AppName[1];
}




char *GetFullPathFromSpec(char *CompletePath, FSSpec *Spec, OSErr *err)
{
Handle hFullPath;
short len;

memset(CompletePath, 0, sizeof(CompletePath));
*err = 0;
*err = FSpGetFullPath(Spec, &len, &hFullPath);
printerr("FSpGetFullPath:",(*err != -43) && (*err != 0),*err,__LINE__,__FILE__,"");
strncpy(CompletePath,*hFullPath,len);
DisposeHandle(hFullPath);   /* we don't need it any more */

CompletePath[len] = 0x00;  /* make c-string */
printerr("Warning path length exceeds limit: ",len >= NAME_MAX ,len,__LINE__,__FILE__," chars ");

return CompletePath;
}
